@@ -25,13 +25,13 @@ namespace
 // notified, if the slot_rep is deleted when they call disconnect().
 struct destroy_notify_struct
 {
-  destroy_notify_struct() : deleted_(false) { }
+  destroy_notify_struct() noexcept : deleted_(false) { }
 
-  static void* notify(void* data)
+  static void* notify(void* data) noexcept
   {
     auto self_ = reinterpret_cast<destroy_notify_struct*>(data);
     self_->deleted_ = true;
-    return 0;
+    return nullptr;
   }
 
   bool deleted_;
@@ -55,23 +55,22 @@ void slot_rep::operator delete(void* p)
 }
 #endif
 
-//TODO: When we can break API: Is it necessary to invalidate the slot here?
-// If the parent misbehaves, when the slot is not invalidated, isn't that
-// a problem that should be fixed in the parent?
-// See discussion in https://bugzilla.gnome.org/show_bug.cgi?id=738602
 void slot_rep::disconnect()
 {
+  // Invalidate the slot.
+  // _Must_ be done here because parent_ might defer the actual
+  // destruction of the slot_rep and try to invoke it before that point.
+  // Must be done also for a slot without a parent, according to
+  // https://bugzilla.gnome.org/show_bug.cgi?id=311057
+  // See also https://bugzilla.gnome.org/show_bug.cgi?id=738602
+  call_ = nullptr;
+
   if (parent_)
   {
-    call_ = nullptr;          // Invalidate the slot.
-                        // _Must_ be done here because parent_ might defer the actual
-                        // destruction of the slot_rep and try to invoke it before that point.
     auto data_ = parent_;
-    parent_ = nullptr;        // Just a precaution.
+    parent_ = nullptr;  // Just a precaution.
     (cleanup_)(data_);  // Notify the parent (might lead to destruction of this!).
   }
-  else
-    call_ = nullptr;
 }
 
 //static
@@ -92,17 +91,17 @@ void* slot_rep::notify(void* data)
     self_->destroy(); // Detach the stored functor from the other referred trackables and destroy it.
                       // destroy() might lead to deletion of self_. Bug #564005.
   }
-  return 0;
+  return nullptr;
 }
 
 } // namespace internal
   
-slot_base::slot_base()
+slot_base::slot_base() noexcept
 : rep_(nullptr),
   blocked_(false)
 {}
 
-slot_base::slot_base(rep_type* rep)
+slot_base::slot_base(rep_type* rep) noexcept
 : rep_(rep),
   blocked_(false)
 {}
@@ -125,13 +124,36 @@ slot_base::slot_base(const slot_base& src)
   }
 }
 
-slot_base::slot_base(slot_base&& src) noexcept
-: rep_(std::move(src.rep_)),
-  blocked_(std::move(src.blocked_))
+slot_base::slot_base(slot_base&& src)
+: rep_(nullptr),
+  blocked_(src.blocked_)
 {
-  //Wipe src:
-  src.rep_ = nullptr;
-  src.blocked_ = false;
+  if (src.rep_)
+  {
+    if (src.rep_->parent_)
+    {
+      // src is connected to a parent, e.g. a sigc::signal.
+      // Copy, don't move! See https://bugzilla.gnome.org/show_bug.cgi?id=756484
+
+      //Check call_ so we can ignore invalidated slots.
+      //Otherwise, destroyed bound reference parameters (whose destruction
+      //caused the slot's invalidation) may be used during dup().
+      if (src.rep_->call_)
+        rep_ = src.rep_->dup();
+      else
+        blocked_ = false; //Return the default invalid slot.
+    }
+    else
+    {
+      // src is not connected. Really move src.rep_.
+      src.rep_->notify_callbacks();
+      rep_ = src.rep_;
+
+      //Wipe src:
+      src.rep_ = nullptr;
+      src.blocked_ = false;
+    }
+  }
 }
 
 slot_base::~slot_base()
@@ -140,7 +162,7 @@ slot_base::~slot_base()
     delete rep_;
 }
 
-slot_base::operator bool() const
+slot_base::operator bool() const noexcept
 {
   return rep_ != nullptr;
 }
@@ -197,24 +219,49 @@ slot_base& slot_base::operator=(const slot_base& src)
   return *this;
 }
 
-slot_base& slot_base::operator=(slot_base&& src) noexcept
+slot_base& slot_base::operator=(slot_base&& src)
 {
   if (src.rep_ == rep_)
+  {
+    blocked_ = src.blocked_;
     return *this;
- 
-  delete_rep_with_check();
+  }
 
-  rep_ = std::move(src.rep_);
-  blocked_ = std::move(src.blocked_);
+  if (src.empty())
+  {
+    delete_rep_with_check();
+    return *this;
+  }
 
-  //Wipe src:
-  src.rep_ = nullptr;
-  src.blocked_ = false;
+  blocked_ = src.blocked_;
+  internal::slot_rep* new_rep_ = nullptr;
+  if (src.rep_->parent_)
+  {
+    // src is connected to a parent, e.g. a sigc::signal.
+    // Copy, don't move! See https://bugzilla.gnome.org/show_bug.cgi?id=756484
+    new_rep_ = src.rep_->dup();
+  }
+  else
+  {
+    // src is not connected. Really move src.rep_.
+    src.rep_->notify_callbacks();
+    new_rep_ = src.rep_;
 
+    //Wipe src:
+    src.rep_ = nullptr;
+    src.blocked_ = false;
+  }
+
+  if (rep_) // Silently exchange the slot_rep.
+  {
+    new_rep_->set_parent(rep_->parent_, rep_->cleanup_);
+    delete rep_; // Calls destroy(), but does not call disconnect().
+  }
+  rep_ = new_rep_;
   return *this;
 }
 
-void slot_base::set_parent(void* parent, void* (*cleanup)(void*)) const
+void slot_base::set_parent(void* parent, void* (*cleanup)(void*)) const noexcept
 {
   if (rep_)
     rep_->set_parent(parent, cleanup);
@@ -232,14 +279,14 @@ void slot_base::remove_destroy_notify_callback(void* data) const
     rep_->remove_destroy_notify_callback(data);
 }
 
-bool slot_base::block(bool should_block)
+bool slot_base::block(bool should_block) noexcept
 {
   bool old = blocked_;
   blocked_ = should_block;
   return old;
 }
 
-bool slot_base::unblock()
+bool slot_base::unblock() noexcept
 {
   return block(false);
 }
